@@ -33,6 +33,8 @@ private:
 	static void OBSMediaPlay(void *data, calldata_t *calldata);
 	static void OBSMediaPause(void *data, calldata_t *calldata);
 	static void OBSMediaStarted(void *data, calldata_t *calldata);
+	static void OBSMediaNext(void *data, calldata_t *calldata);
+	static void OBSMediaPrevious(void *data, calldata_t *calldata);
 
 	std::unique_ptr<Ui_MediaControls> ui;
 
@@ -61,6 +63,8 @@ private slots:
 	void MoveSliderFoward(int seconds = 5);
 	void MoveSliderBackwards(int seconds = 5);
 
+	void UpdateSlideCounter();
+
 public slots:
 	void PlayMedia();
 	void PauseMedia();
