@@ -94,3 +94,32 @@ QAccessible::Role VolumeAccessibleInterface::role() const
 {
 	return QAccessible::Role::Slider;
 }
+
+void SliderIgnoreClick::mousePressEvent(QMouseEvent *event)
+{
+	QStyleOptionSlider styleOption;
+	initStyleOption(&styleOption);
+	QRect handle = style()->subControlRect(QStyle::CC_Slider, &styleOption,
+					       QStyle::SC_SliderHandle, this);
+	if (handle.contains(event->position().toPoint())) {
+		SliderIgnoreScroll::mousePressEvent(event);
+		dragging = true;
+	} else {
+		event->accept();
+	}
+}
+
+void SliderIgnoreClick::mouseReleaseEvent(QMouseEvent *event)
+{
+	dragging = false;
+	SliderIgnoreScroll::mouseReleaseEvent(event);
+}
+
+void SliderIgnoreClick::mouseMoveEvent(QMouseEvent *event)
+{
+	if (dragging) {
+		SliderIgnoreScroll::mouseMoveEvent(event);
+	} else {
+		event->accept();
+	}
+}
