@@ -5,6 +5,7 @@
 #include <QInputEvent>
 #include <QtCore/QObject>
 #include <QAccessibleWidget>
+#include <QStyleOptionSlider>
 
 class SliderIgnoreScroll : public QSlider {
 	Q_OBJECT
@@ -49,3 +50,20 @@ protected:
 	virtual QAccessible::Role role() const override;
 	virtual QString text(QAccessible::Text t) const override;
 };
+
+class SliderIgnoreClick : public SliderIgnoreScroll {
+public:
+	inline SliderIgnoreClick(Qt::Orientation orientation,
+				 QWidget *parent = nullptr)
+		: SliderIgnoreScroll(orientation, parent)
+	{
+	}
+
+protected:
+	virtual void mousePressEvent(QMouseEvent *event) override;
+	virtual void mouseReleaseEvent(QMouseEvent *event) override;
+	virtual void mouseMoveEvent(QMouseEvent *event) override;
+
+private:
+	bool dragging = false;
+};
