@@ -24,7 +24,7 @@ OBSAbout::OBSAbout(QWidget *parent) : QDialog(parent), ui(new Ui::OBSAbout)
 		bitness = " (64 bit)";
 
 #ifdef HAVE_OBSCONFIG_H
-	ver += OBS_VERSION;
+	ver += obs_get_version_string();
 #else
 	ver += LIBOBS_API_MAJOR_VER + "." + LIBOBS_API_MINOR_VER + "." +
 	       LIBOBS_API_PATCH_VER;
@@ -61,9 +61,12 @@ OBSAbout::OBSAbout(QWidget *parent) : QDialog(parent), ui(new Ui::OBSAbout)
 	ui->license->setProperty("themeID", "aboutHLayout");
 	ui->info->setProperty("themeID", "aboutInfo");
 
-	connect(ui->about, SIGNAL(clicked()), this, SLOT(ShowAbout()));
-	connect(ui->authors, SIGNAL(clicked()), this, SLOT(ShowAuthors()));
-	connect(ui->license, SIGNAL(clicked()), this, SLOT(ShowLicense()));
+	connect(ui->about, &ClickableLabel::clicked, this,
+		&OBSAbout::ShowAbout);
+	connect(ui->authors, &ClickableLabel::clicked, this,
+		&OBSAbout::ShowAuthors);
+	connect(ui->license, &ClickableLabel::clicked, this,
+		&OBSAbout::ShowLicense);
 
 	QPointer<OBSAbout> about(this);
 
@@ -74,10 +77,8 @@ OBSAbout::OBSAbout(QWidget *parent) : QDialog(parent), ui(new Ui::OBSAbout)
 			"application/json");
 		QObject::connect(thread, &RemoteTextThread::Result, main,
 				 &OBSBasic::UpdatePatronJson);
-		QObject::connect(
-			thread,
-			SIGNAL(Result(const QString &, const QString &)), this,
-			SLOT(ShowAbout()));
+		QObject::connect(thread, &RemoteTextThread::Result, this,
+				 &OBSAbout::ShowAbout);
 		main->patronJsonThread.reset(thread);
 		thread->start();
 	} else {
@@ -133,8 +134,9 @@ void OBSAbout::ShowAbout()
 void OBSAbout::ShowAuthors()
 {
 	std::string path;
-	QString error = "Error! File could not be read.\n\n \
-		Go to: https://github.com/obsproject/obs-studio/blob/master/AUTHORS";
+	QString error =
+		QTStr("About.Error")
+			.arg("https://github.com/obsproject/obs-studio/blob/master/AUTHORS");
 
 #ifdef __APPLE__
 	if (!GetDataFilePath("AUTHORS", path)) {
@@ -160,8 +162,9 @@ void OBSAbout::ShowAuthors()
 void OBSAbout::ShowLicense()
 {
 	std::string path;
-	QString error = "Error! File could not be read.\n\n \
-		Go to: https://github.com/obsproject/obs-studio/blob/master/COPYING";
+	QString error =
+		QTStr("About.Error")
+			.arg("https://github.com/obsproject/obs-studio/blob/master/COPYING");
 
 	if (!GetDataFilePath("license/gplv2.txt", path)) {
 		ui->textBrowser->setPlainText(error);
