@@ -85,13 +85,13 @@ public:
 /* ------------------------------------------------------------------------- */
 
 #define TEST_STR(x) "Basic.AutoConfig.TestPage." x
-#define SUBTITLE_TESTING TEST_STR("Subtitle.Testing")
-#define SUBTITLE_COMPLETE TEST_STR("Subtitle.Complete")
+#define SUBTITLE_TESTING TEST_STR("SubTitle.Testing")
+#define SUBTITLE_COMPLETE TEST_STR("SubTitle.Complete")
 #define TEST_BW TEST_STR("TestingBandwidth")
+#define TEST_BW_NO_OUTPUT TEST_STR("TestingBandwidth.NoOutput")
 #define TEST_BW_CONNECTING TEST_STR("TestingBandwidth.Connecting")
 #define TEST_BW_CONNECT_FAIL TEST_STR("TestingBandwidth.ConnectFailed")
 #define TEST_BW_SERVER TEST_STR("TestingBandwidth.Server")
-#define TEST_RES TEST_STR("TestingRes")
 #define TEST_RES_VAL TEST_STR("TestingRes.Resolution")
 #define TEST_RES_FAIL TEST_STR("TestingRes.Fail")
 #define TEST_SE TEST_STR("TestingStreamEncoder")
@@ -156,6 +156,23 @@ static inline void string_depad_key(string &key)
 
 const char *FindAudioEncoderFromCodec(const char *type);
 
+static inline bool can_use_output(const char *prot, const char *output,
+				  const char *prot_test1,
+				  const char *prot_test2 = nullptr)
+{
+	return (strcmp(prot, prot_test1) == 0 ||
+		(prot_test2 && strcmp(prot, prot_test2) == 0)) &&
+	       (obs_get_output_flags(output) & OBS_OUTPUT_SERVICE) != 0;
+}
+
+static bool return_first_id(void *data, const char *id)
+{
+	const char **output = (const char **)data;
+
+	*output = id;
+	return false;
+}
+
 void AutoConfigTestPage::TestBandwidthThread()
 {
 	bool connected = false;
@@ -232,6 +249,10 @@ void AutoConfigTestPage::TestBandwidthThread()
 		config_get_string(main->Config(), "Output", "BindIP");
 	obs_data_set_string(output_settings, "bind_ip", bind_ip);
 
+	const char *ip_family =
+		config_get_string(main->Config(), "Output", "IPFamily");
+	obs_data_set_string(output_settings, "ip_family", ip_family);
+
 	/* -----------------------------------*/
 	/* determine which servers to test    */
 
@@ -269,9 +290,37 @@ void AutoConfigTestPage::TestBandwidthThread()
 	/* -----------------------------------*/
 	/* create output                      */
 
-	const char *output_type = obs_service_get_output_type(service);
-	if (!output_type)
-		output_type = "rtmp_output";
+	/* Check if the service has a preferred output type */
+	const char *output_type =
+		obs_service_get_preferred_output_type(service);
+	if (!output_type ||
+	    (obs_get_output_flags(output_type) & OBS_OUTPUT_SERVICE) == 0) {
+		/* Otherwise, prefer first-party output types */
+		const char *protocol = obs_service_get_protocol(service);
+
+		if (can_use_output(protocol, "rtmp_output", "RTMP", "RTMPS")) {
+			output_type = "rtmp_output";
+		} else if (can_use_output(protocol, "ffmpeg_hls_muxer",
+					  "HLS")) {
+			output_type = "ffmpeg_hls_muxer";
+		} else if (can_use_output(protocol, "ffmpeg_mpegts_muxer",
+					  "SRT", "RIST")) {
+			output_type = "ffmpeg_mpegts_muxer";
+		}
+
+		/* If third-party protocol, use the first enumerated type */
+		if (!output_type)
+			obs_enum_output_types_with_protocol(
+				protocol, &output_type, return_first_id);
+
+		/* If none, fail */
+		if (!output_type) {
+			QMetaObject::invokeMethod(
+				this, "Failure",
+				Q_ARG(QString, QTStr(TEST_BW_NO_OUTPUT)));
+			return;
+		}
+	}
 
 	OBSOutputAutoRelease output =
 		obs_output_create(output_type, "test_stream", nullptr, nullptr);
@@ -489,7 +538,10 @@ struct Result {
 	int fps_den;
 
 	inline Result(int cx_, int cy_, int fps_num_, int fps_den_)
-		: cx(cx_), cy(cy_), fps_num(fps_num_), fps_den(fps_den_)
+		: cx(cx_),
+		  cy(cy_),
+		  fps_num(fps_num_),
+		  fps_den(fps_den_)
 	{
 	}
 };
@@ -902,6 +954,16 @@ void AutoConfigTestPage::TestStreamEncoderThread()
 		wiz->streamingEncoder = AutoConfig::Encoder::x264;
 	}
 
+#ifdef __linux__
+	// On linux CBR rate control is not guaranteed so fallback to x264.
+	if (wiz->streamingEncoder == AutoConfig::Encoder::QSV) {
+		wiz->streamingEncoder = AutoConfig::Encoder::x264;
+		if (!TestSoftwareEncoding()) {
+			return;
+		}
+	}
+#endif
+
 	if (preferHardware && !softwareTested && wiz->hardwareEncodingAvailable)
 		FindIdealHardwareResolution();
 
@@ -1172,7 +1234,8 @@ void AutoConfigTestPage::Progress(int percentage)
 }
 
 AutoConfigTestPage::AutoConfigTestPage(QWidget *parent)
-	: QWizardPage(parent), ui(new Ui_AutoConfigTestPage)
+	: QWizardPage(parent),
+	  ui(new Ui_AutoConfigTestPage)
 {
 	ui->setupUi(this);
 	setTitle(QTStr("Basic.AutoConfig.TestPage"));
