@@ -8,8 +8,16 @@ extern QCef *cef;
 extern QCefCookieManager *panel_cookies;
 
 class BrowserDock : public OBSDock {
+private:
+	QString title;
+
 public:
 	inline BrowserDock() : OBSDock() { setAttribute(Qt::WA_NativeWindow); }
+	inline BrowserDock(const QString &title_) : OBSDock(title_)
+	{
+		title = title_;
+		setAttribute(Qt::WA_NativeWindow);
+	}
 
 	QScopedPointer<QCefWidget> cefWidget;
 
@@ -19,5 +27,8 @@ public:
 		cefWidget.reset(widget_);
 	}
 
+	inline void setTitle(const QString &title_) { title = title_; }
+
 	void closeEvent(QCloseEvent *event) override;
+	void showEvent(QShowEvent *event) override;
 };
