@@ -34,3 +34,8 @@ void OBSDock::closeEvent(QCloseEvent *event)
 
 	QDockWidget::closeEvent(event);
 }
+
+void OBSDock::showEvent(QShowEvent *event)
+{
+	QDockWidget::showEvent(event);
+}
