@@ -56,7 +56,9 @@ enum RemuxEntryRole { EntryStateRole = Qt::UserRole, NewPathsToProcessRole };
 
 RemuxEntryPathItemDelegate::RemuxEntryPathItemDelegate(
 	bool isOutput, const QString &defaultPath)
-	: QStyledItemDelegate(), isOutput(isOutput), defaultPath(defaultPath)
+	: QStyledItemDelegate(),
+	  isOutput(isOutput),
+	  defaultPath(defaultPath)
 {
 }
 
@@ -112,8 +114,8 @@ QWidget *RemuxEntryPathItemDelegate::createEditor(
 				    QSizePolicy::ControlType::LineEdit));
 		layout->addWidget(text);
 
-		QObject::connect(text, SIGNAL(editingFinished()), this,
-				 SLOT(updateText()));
+		QObject::connect(text, &QLineEdit::editingFinished, this,
+				 &RemuxEntryPathItemDelegate::updateText);
 
 		QToolButton *browseButton = new QToolButton();
 		browseButton->setText("...");
@@ -205,8 +207,7 @@ void RemuxEntryPathItemDelegate::paint(QPainter *painter,
 
 void RemuxEntryPathItemDelegate::handleBrowse(QWidget *container)
 {
-	QString OutputPattern = "(*.mp4 *.flv *.mov *.mkv *.ts *.m3u8)";
-	QString InputPattern = "(*.flv *.mov *.mkv *.ts *.m3u8)";
+	QString ExtensionPattern = "(*.mp4 *.flv *.mov *.mkv *.ts *.m3u8)";
 
 	QLineEdit *text = container->findChild<QLineEdit *>();
 
@@ -218,7 +219,7 @@ void RemuxEntryPathItemDelegate::handleBrowse(QWidget *container)
 	if (isOutput) {
 		QString newPath = SaveFile(container,
 					   QTStr("Remux.SelectTarget"),
-					   currentPath, OutputPattern);
+					   currentPath, ExtensionPattern);
 
 		if (!newPath.isEmpty()) {
 			container->setProperty(PATH_LIST_PROP,
@@ -229,12 +230,16 @@ void RemuxEntryPathItemDelegate::handleBrowse(QWidget *container)
 		QStringList paths = OpenFiles(
 			container, QTStr("Remux.SelectRecording"), currentPath,
 			QTStr("Remux.OBSRecording") + QString(" ") +
-				InputPattern);
+				ExtensionPattern);
 
 		if (!paths.empty()) {
 			container->setProperty(PATH_LIST_PROP, paths);
 			isSet = true;
 		}
+#ifdef __APPLE__
+		// TODO: Revisit when QTBUG-42661 is fixed
+		container->window()->raise();
+#endif
 	}
 
 	if (isSet)
@@ -349,7 +354,7 @@ bool RemuxQueueModel::setData(const QModelIndex &index, const QVariant &value,
 				endRemoveRows();
 			}
 		} else {
-			if (pathList.size() > 1 &&
+			if (pathList.size() >= 1 &&
 			    index.row() < queue.length()) {
 				queue[index.row()].sourcePath = pathList[0];
 				checkInputPath(index.row());
@@ -367,6 +372,7 @@ bool RemuxQueueModel::setData(const QModelIndex &index, const QVariant &value,
 				for (QString path : pathList) {
 					RemuxQueueEntry entry;
 					entry.sourcePath = path;
+					entry.state = RemuxEntryState::Empty;
 
 					queue.insert(row, entry);
 					row++;
@@ -386,6 +392,7 @@ bool RemuxQueueModel::setData(const QModelIndex &index, const QVariant &value,
 		if (!path.isEmpty()) {
 			RemuxQueueEntry entry;
 			entry.sourcePath = path;
+			entry.state = RemuxEntryState::Empty;
 
 			beginInsertRows(QModelIndex(), queue.length() + 1,
 					queue.length() + 1);
@@ -469,10 +476,18 @@ void RemuxQueueModel::checkInputPath(int row)
 		else
 			entry.state = RemuxEntryState::InvalidPath;
 
+		QString newExt = ".mp4";
+		QString suffix = fileInfo.suffix();
+
+		if (suffix.contains("mov", Qt::CaseInsensitive) ||
+		    suffix.contains("mp4", Qt::CaseInsensitive)) {
+			newExt = ".remuxed." + suffix;
+		}
+
 		if (entry.state == RemuxEntryState::Ready)
 			entry.targetPath = QDir::toNativeSeparators(
 				fileInfo.path() + QDir::separator() +
-				fileInfo.completeBaseName() + ".mp4");
+				fileInfo.completeBaseName() + newExt);
 	}
 
 	if (entry.state == RemuxEntryState::Ready && isProcessing)
@@ -685,35 +700,30 @@ OBSRemux::OBSRemux(const char *path, QWidget *parent, bool autoRemux_)
 		->setText(QTStr("Remux.ClearAll"));
 	ui->buttonBox->button(QDialogButtonBox::Reset)->setDisabled(true);
 
-	connect(ui->buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()),
-		this, SLOT(beginRemux()));
+	connect(ui->buttonBox->button(QDialogButtonBox::Ok),
+		&QPushButton::clicked, this, &OBSRemux::beginRemux);
 	connect(ui->buttonBox->button(QDialogButtonBox::Reset),
-		SIGNAL(clicked()), this, SLOT(clearFinished()));
+		&QPushButton::clicked, this, &OBSRemux::clearFinished);
 	connect(ui->buttonBox->button(QDialogButtonBox::RestoreDefaults),
-		SIGNAL(clicked()), this, SLOT(clearAll()));
+		&QPushButton::clicked, this, &OBSRemux::clearAll);
 	connect(ui->buttonBox->button(QDialogButtonBox::Close),
-		SIGNAL(clicked()), this, SLOT(close()));
+		&QPushButton::clicked, this, &OBSRemux::close);
 
 	worker->moveToThread(&remuxer);
 	remuxer.start();
 
-	//gcc-4.8 can't use QPointer<RemuxWorker> below
-	RemuxWorker *worker_ = worker;
-	connect(worker_, &RemuxWorker::updateProgress, this,
+	connect(worker.data(), &RemuxWorker::updateProgress, this,
 		&OBSRemux::updateProgress);
-	connect(&remuxer, &QThread::finished, worker_, &QObject::deleteLater);
-	connect(worker_, &RemuxWorker::remuxFinished, this,
+	connect(&remuxer, &QThread::finished, worker.data(),
+		&QObject::deleteLater);
+	connect(worker.data(), &RemuxWorker::remuxFinished, this,
 		&OBSRemux::remuxFinished);
-	connect(this, &OBSRemux::remux, worker_, &RemuxWorker::remux);
+	connect(this, &OBSRemux::remux, worker.data(), &RemuxWorker::remux);
 
-	// Guessing the GCC bug mentioned above would also affect
-	// QPointer<RemuxQueueModel>? Unsure.
-	RemuxQueueModel *queueModel_ = queueModel;
-	connect(queueModel_,
-		SIGNAL(rowsInserted(const QModelIndex &, int, int)), this,
-		SLOT(rowCountChanged(const QModelIndex &, int, int)));
-	connect(queueModel_, SIGNAL(rowsRemoved(const QModelIndex &, int, int)),
-		this, SLOT(rowCountChanged(const QModelIndex &, int, int)));
+	connect(queueModel.data(), &RemuxQueueModel::rowsInserted, this,
+		&OBSRemux::rowCountChanged);
+	connect(queueModel.data(), &RemuxQueueModel::rowsRemoved, this,
+		&OBSRemux::rowCountChanged);
 
 	QModelIndex index = queueModel->createIndex(0, 1);
 	QMetaObject::invokeMethod(ui->tableView, "setCurrentIndex",
@@ -930,7 +940,7 @@ void OBSRemux::remuxFinished(bool success)
 	queueModel->finishEntry(success);
 
 	if (autoRemux && autoRemuxFile != "") {
-		QTimer::singleShot(3000, this, SLOT(close()));
+		QTimer::singleShot(3000, this, &OBSRemux::close);
 
 		OBSBasic *main = OBSBasic::Get();
 		main->ShowStatusBarMessage(
