@@ -1,5 +1,5 @@
 /******************************************************************************
-    Copyright (C) 2013 by Hugh Bailey <obs.jim@gmail.com>
+    Copyright (C) 2023 by Lain Bailey <lain@obsproject.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,9 @@
 
 // #define DISASSEMBLE_SHADERS
 
+typedef HRESULT(WINAPI *pD3DCreateBlob)(_In_ SIZE_T Size,
+					_Out_ ID3DBlob **ppBlob);
+
 struct shader_var;
 struct shader_sampler;
 struct gs_vertex_shader;
@@ -143,7 +146,7 @@ ConvertGSTextureFormatViewLinear(gs_color_format format)
 
 static inline gs_color_format ConvertDXGITextureFormat(DXGI_FORMAT format)
 {
-	switch ((unsigned long)format) {
+	switch (format) {
 	case DXGI_FORMAT_A8_UNORM:
 		return GS_A8;
 	case DXGI_FORMAT_R8_UNORM:
@@ -476,13 +479,16 @@ struct gs_texture : gs_obj {
 
 	inline gs_texture(gs_texture_type type, uint32_t levels,
 			  gs_color_format format)
-		: type(type), levels(levels), format(format)
+		: type(type),
+		  levels(levels),
+		  format(format)
 	{
 	}
 
 	inline gs_texture(gs_device *device, gs_type obj_type,
 			  gs_texture_type type)
-		: gs_obj(device, obj_type), type(type)
+		: gs_obj(device, obj_type),
+		  type(type)
 	{
 	}
 
@@ -632,7 +638,9 @@ struct gs_zstencil_buffer : gs_obj {
 	}
 
 	inline gs_zstencil_buffer()
-		: width(0), height(0), dxgiFormat(DXGI_FORMAT_UNKNOWN)
+		: width(0),
+		  height(0),
+		  dxgiFormat(DXGI_FORMAT_UNKNOWN)
 	{
 	}
 
@@ -693,7 +701,8 @@ struct ShaderError {
 	HRESULT hr;
 
 	inline ShaderError(const ComPtr<ID3D10Blob> &errors, HRESULT hr)
-		: errors(errors), hr(hr)
+		: errors(errors),
+		  hr(hr)
 	{
 	}
 };
@@ -717,7 +726,9 @@ struct gs_shader : gs_obj {
 
 	inline gs_shader(gs_device_t *device, gs_type obj_type,
 			 gs_shader_type type)
-		: gs_obj(device, obj_type), type(type), constantSize(0)
+		: gs_obj(device, obj_type),
+		  type(type),
+		  constantSize(0)
 	{
 	}
 
@@ -730,7 +741,8 @@ struct ShaderSampler {
 
 	inline ShaderSampler(const char *name, gs_device_t *device,
 			     gs_sampler_info *info)
-		: name(name), sampler(device, info)
+		: name(name),
+		  sampler(device, info)
 	{
 	}
 };
@@ -779,6 +791,9 @@ struct gs_vertex_shader : gs_shader {
 struct gs_duplicator : gs_obj {
 	ComPtr<IDXGIOutputDuplication> duplicator;
 	gs_texture_2d *texture;
+	bool hdr = false;
+	enum gs_color_space color_space = GS_CS_SRGB;
+	float sdr_white_nits = 80.f;
 	int idx;
 	long refs;
 	bool updated;
@@ -891,7 +906,8 @@ struct SavedBlendState : BlendState {
 	inline void Release() { state.Release(); }
 
 	inline SavedBlendState(const BlendState &val, D3D11_BLEND_DESC &desc)
-		: BlendState(val), bd(desc)
+		: BlendState(val),
+		  bd(desc)
 	{
 	}
 };
@@ -903,7 +919,10 @@ struct StencilSide {
 	gs_stencil_op_type zpass;
 
 	inline StencilSide()
-		: test(GS_ALWAYS), fail(GS_KEEP), zfail(GS_KEEP), zpass(GS_KEEP)
+		: test(GS_ALWAYS),
+		  fail(GS_KEEP),
+		  zfail(GS_KEEP),
+		  zpass(GS_KEEP)
 	{
 	}
 };
@@ -943,7 +962,8 @@ struct SavedZStencilState : ZStencilState {
 
 	inline SavedZStencilState(const ZStencilState &val,
 				  D3D11_DEPTH_STENCIL_DESC desc)
-		: ZStencilState(val), dsd(desc)
+		: ZStencilState(val),
+		  dsd(desc)
 	{
 	}
 };
@@ -970,7 +990,8 @@ struct SavedRasterState : RasterState {
 
 	inline SavedRasterState(const RasterState &val,
 				D3D11_RASTERIZER_DESC &desc)
-		: RasterState(val), rd(desc)
+		: RasterState(val),
+		  rd(desc)
 	{
 	}
 };
@@ -982,9 +1003,13 @@ struct mat4float {
 struct gs_monitor_color_info {
 	bool hdr;
 	UINT bits_per_color;
+	ULONG sdr_white_nits;
 
-	gs_monitor_color_info(bool hdr, int bits_per_color)
-		: hdr(hdr), bits_per_color(bits_per_color)
+	gs_monitor_color_info(bool hdr, int bits_per_color,
+			      ULONG sdr_white_nits)
+		: hdr(hdr),
+		  bits_per_color(bits_per_color),
+		  sdr_white_nits(sdr_white_nits)
 	{
 	}
 };
@@ -997,6 +1022,7 @@ struct gs_device {
 	uint32_t adpIdx = 0;
 	bool nv12Supported = false;
 	bool p010Supported = false;
+	bool fastClearSupported = false;
 
 	gs_texture_2d *curRenderTarget = nullptr;
 	gs_zstencil_buffer *curZStencilBuffer = nullptr;
@@ -1030,6 +1056,7 @@ struct gs_device {
 	D3D11_PRIMITIVE_TOPOLOGY curToplogy;
 
 	pD3DCompile d3dCompile = nullptr;
+	pD3DCreateBlob d3dCreateBlob = nullptr;
 #ifdef DISASSEMBLE_SHADERS
 	pD3DDisassemble d3dDisassemble = nullptr;
 #endif
@@ -1061,9 +1088,9 @@ struct gs_device {
 
 	void LoadVertexBufferData();
 
-	inline void CopyTex(ID3D11Texture2D *dst, uint32_t dst_x,
-			    uint32_t dst_y, gs_texture_t *src, uint32_t src_x,
-			    uint32_t src_y, uint32_t src_w, uint32_t src_h);
+	void CopyTex(ID3D11Texture2D *dst, uint32_t dst_x, uint32_t dst_y,
+		     gs_texture_t *src, uint32_t src_x, uint32_t src_y,
+		     uint32_t src_w, uint32_t src_h);
 
 	void UpdateViewProjMatrix();
 
@@ -1073,6 +1100,8 @@ struct gs_device {
 
 	bool HasBadNV12Output();
 
+	gs_monitor_color_info GetMonitorColorInfo(HMONITOR hMonitor);
+
 	gs_device(uint32_t adapterIdx);
 	~gs_device();
 };
