@@ -8,8 +8,8 @@
 #include "wasapi-output.h"
 
 #define ACTUALLY_DEFINE_GUID(name, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
-	EXTERN_C const GUID DECLSPEC_SELECTANY name = {                       \
-		l, w1, w2, {b1, b2, b3, b4, b5, b6, b7, b8}}
+	EXTERN_C const GUID DECLSPEC_SELECTANY                                \
+		name = {l, w1, w2, {b1, b2, b3, b4, b5, b6, b7, b8}}
 
 #define do_log(level, format, ...)                      \
 	blog(level, "[audio monitoring: '%s'] " format, \
