@@ -1,5 +1,5 @@
 /******************************************************************************
-    Copyright (C) 2014 by Hugh Bailey <obs.jim@gmail.com>
+    Copyright (C) 2023 by Lain Bailey <lain@obsproject.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,21 +27,21 @@
 /*
  * Increment if major breaking API changes
  */
-#define LIBOBS_API_MAJOR_VER 29
+#define LIBOBS_API_MAJOR_VER 30
 
 /*
  * Increment if backward-compatible additions
  *
  * Reset to zero each major version
  */
-#define LIBOBS_API_MINOR_VER 1
+#define LIBOBS_API_MINOR_VER 0
 
 /*
  * Increment if backward-compatible bug fix
  *
  * Reset to zero each major or minor version
  */
-#define LIBOBS_API_PATCH_VER 3
+#define LIBOBS_API_PATCH_VER 2
 
 #define MAKE_SEMANTIC_VERSION(major, minor, patch) \
 	((major << 24) | (minor << 16) | patch)
@@ -58,15 +58,7 @@
 #define OBS_INSTALL_PREFIX ""
 #define OBS_PLUGIN_DESTINATION "obs-plugins"
 #define OBS_RELATIVE_PREFIX "../../"
-#define OBS_RELEASE_CANDIDATE_MAJOR 0
-#define OBS_RELEASE_CANDIDATE_MINOR 0
-#define OBS_RELEASE_CANDIDATE_PATCH 0
-#define OBS_RELEASE_CANDIDATE_VER 0
 #define OBS_RELEASE_CANDIDATE 0
-#define OBS_BETA_MAJOR 0
-#define OBS_BETA_MINOR 0
-#define OBS_BETA_PATCH 0
-#define OBS_BETA_VER 0
 #define OBS_BETA 0
 #endif
 
