@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015 Hugh Bailey <obs.jim@gmail.com>
+ * Copyright (c) 2023 Lain Bailey <lain@obsproject.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -18,17 +18,27 @@
 
 static inline long os_atomic_inc_long(volatile long *val)
 {
-	return __sync_add_and_fetch(val, 1);
+	return __atomic_add_fetch(val, 1, __ATOMIC_SEQ_CST);
 }
 
 static inline long os_atomic_dec_long(volatile long *val)
 {
-	return __sync_sub_and_fetch(val, 1);
+	return __atomic_sub_fetch(val, 1, __ATOMIC_SEQ_CST);
+}
+
+static inline void os_atomic_store_long(volatile long *ptr, long val)
+{
+	__atomic_store_n(ptr, val, __ATOMIC_SEQ_CST);
 }
 
 static inline long os_atomic_set_long(volatile long *ptr, long val)
 {
-	return __sync_lock_test_and_set(ptr, val);
+	return __atomic_exchange_n(ptr, val, __ATOMIC_SEQ_CST);
+}
+
+static inline long os_atomic_exchange_long(volatile long *ptr, long val)
+{
+	return os_atomic_set_long(ptr, val);
 }
 
 static inline long os_atomic_load_long(const volatile long *ptr)
@@ -39,12 +49,30 @@ static inline long os_atomic_load_long(const volatile long *ptr)
 static inline bool os_atomic_compare_swap_long(volatile long *val, long old_val,
 					       long new_val)
 {
-	return __sync_bool_compare_and_swap(val, old_val, new_val);
+	return __atomic_compare_exchange_n(val, &old_val, new_val, false,
+					   __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
+}
+
+static inline bool os_atomic_compare_exchange_long(volatile long *val,
+						   long *old_val, long new_val)
+{
+	return __atomic_compare_exchange_n(val, old_val, new_val, false,
+					   __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
+}
+
+static inline void os_atomic_store_bool(volatile bool *ptr, bool val)
+{
+	__atomic_store_n(ptr, val, __ATOMIC_SEQ_CST);
 }
 
 static inline bool os_atomic_set_bool(volatile bool *ptr, bool val)
 {
-	return __sync_lock_test_and_set(ptr, val);
+	return __atomic_exchange_n(ptr, val, __ATOMIC_SEQ_CST);
+}
+
+static inline bool os_atomic_exchange_bool(volatile bool *ptr, bool val)
+{
+	return os_atomic_set_bool(ptr, val);
 }
 
 static inline bool os_atomic_load_bool(const volatile bool *ptr)
