@@ -4,7 +4,8 @@
 
 DeckLinkOutput::DeckLinkOutput(obs_output_t *output,
 			       DeckLinkDeviceDiscovery *discovery_)
-	: DecklinkBase(discovery_), output(output)
+	: DecklinkBase(discovery_),
+	  output(output)
 {
 	discovery->AddCallback(DeckLinkOutput::DevicesChanged, this);
 }
@@ -82,9 +83,9 @@ obs_output_t *DeckLinkOutput::GetOutput(void) const
 	return output;
 }
 
-void DeckLinkOutput::DisplayVideoFrame(video_data *frame)
+void DeckLinkOutput::UpdateVideoFrame(video_data *frame)
 {
-	instance->DisplayVideoFrame(frame);
+	instance->UpdateVideoFrame(frame);
 }
 
 void DeckLinkOutput::WriteAudio(audio_data *frames)
