@@ -71,6 +71,8 @@ struct image_file_data {
 	obs_source_t *source;
 };
 
+typedef DARRAY(struct image_file_data) image_file_array_t;
+
 enum behavior {
 	BEHAVIOR_STOP_RESTART,
 	BEHAVIOR_PAUSE_UNPAUSE,
@@ -94,6 +96,7 @@ struct slideshow {
 	uint32_t tr_speed;
 	const char *tr_name;
 	obs_source_t *transition;
+	calldata_t cd;
 
 	float elapsed;
 	size_t cur_item;
@@ -103,7 +106,7 @@ struct slideshow {
 	uint64_t mem_usage;
 
 	pthread_mutex_t mutex;
-	DARRAY(struct image_file_data) files;
+	image_file_array_t files;
 
 	enum behavior behavior;
 
@@ -139,18 +142,15 @@ static obs_source_t *get_transition(struct slideshow *ss)
 	return tr;
 }
 
-static obs_source_t *get_source(struct darray *array, const char *path)
+static obs_source_t *get_source(image_file_array_t *files, const char *path)
 {
-	DARRAY(struct image_file_data) files;
 	obs_source_t *source = NULL;
 
-	files.da = *array;
-
-	for (size_t i = 0; i < files.num; i++) {
-		const char *cur_path = files.array[i].path;
+	for (size_t i = 0; i < files->num; i++) {
+		const char *cur_path = files->array[i].path;
 
 		if (strcmp(path, cur_path) == 0) {
-			source = obs_source_get_ref(files.array[i].source);
+			source = obs_source_get_ref(files->array[i].source);
 			break;
 		}
 	}
@@ -171,17 +171,14 @@ static obs_source_t *create_source_from_file(const char *file)
 	return source;
 }
 
-static void free_files(struct darray *array)
+static void free_files(image_file_array_t *files)
 {
-	DARRAY(struct image_file_data) files;
-	files.da = *array;
-
-	for (size_t i = 0; i < files.num; i++) {
-		bfree(files.array[i].path);
-		obs_source_release(files.array[i].source);
+	for (size_t i = 0; i < files->num; i++) {
+		bfree(files->array[i].path);
+		obs_source_release(files->array[i].source);
 	}
 
-	da_free(files);
+	da_free(*files);
 }
 
 static inline size_t random_file(struct slideshow *ss)
@@ -204,21 +201,18 @@ static const char *ss_getname(void *unused)
 	return obs_module_text("SlideShow");
 }
 
-static void add_file(struct slideshow *ss, struct darray *array,
+static void add_file(struct slideshow *ss, image_file_array_t *new_files,
 		     const char *path, uint32_t *cx, uint32_t *cy)
 {
-	DARRAY(struct image_file_data) new_files;
 	struct image_file_data data;
 	obs_source_t *new_source;
 
-	new_files.da = *array;
-
 	pthread_mutex_lock(&ss->mutex);
-	new_source = get_source(&ss->files.da, path);
+	new_source = get_source(&ss->files, path);
 	pthread_mutex_unlock(&ss->mutex);
 
 	if (!new_source)
-		new_source = get_source(&new_files.da, path);
+		new_source = get_source(new_files, path);
 	if (!new_source)
 		new_source = create_source_from_file(path);
 
@@ -228,7 +222,7 @@ static void add_file(struct slideshow *ss, struct darray *array,
 
 		data.path = bstrdup(path);
 		data.source = new_source;
-		da_push_back(new_files, &data);
+		da_push_back(*new_files, &data);
 
 		if (new_cx > *cx)
 			*cx = new_cx;
@@ -238,8 +232,6 @@ static void add_file(struct slideshow *ss, struct darray *array,
 		void *source_data = obs_obj_get_data(new_source);
 		ss->mem_usage += image_source_get_memory_usage(source_data);
 	}
-
-	*array = new_files.da;
 }
 
 static bool valid_extension(const char *ext)
@@ -280,12 +272,22 @@ static void do_transition(void *data, bool to_null)
 		set_media_state(ss, OBS_MEDIA_STATE_ENDED);
 		obs_source_media_ended(ss->source);
 	}
+
+	if (valid && !to_null) {
+		calldata_set_int(&ss->cd, "index", ss->cur_item);
+		calldata_set_string(&ss->cd, "path",
+				    ss->files.array[ss->cur_item].path);
+
+		signal_handler_t *sh =
+			obs_source_get_signal_handler(ss->source);
+		signal_handler_signal(sh, "slide_changed", &ss->cd);
+	}
 }
 
 static void ss_update(void *data, obs_data_t *settings)
 {
-	DARRAY(struct image_file_data) new_files;
-	DARRAY(struct image_file_data) old_files;
+	image_file_array_t new_files;
+	image_file_array_t old_files;
 	obs_source_t *new_tr = NULL;
 	obs_source_t *old_tr = NULL;
 	struct slideshow *ss = data;
@@ -375,7 +377,7 @@ static void ss_update(void *data, obs_data_t *settings)
 				dstr_copy(&dir_path, path);
 				dstr_cat_ch(&dir_path, '/');
 				dstr_cat(&dir_path, ent->d_name);
-				add_file(ss, &new_files.da, dir_path.array, &cx,
+				add_file(ss, &new_files, dir_path.array, &cx,
 					 &cy);
 
 				if (ss->mem_usage >= MAX_MEM_USAGE)
@@ -385,7 +387,7 @@ static void ss_update(void *data, obs_data_t *settings)
 			dstr_free(&dir_path);
 			os_closedir(dir);
 		} else {
-			add_file(ss, &new_files.da, path, &cx, &cy);
+			add_file(ss, &new_files, path, &cx, &cy);
 		}
 
 		obs_data_release(item);
@@ -399,8 +401,8 @@ static void ss_update(void *data, obs_data_t *settings)
 
 	pthread_mutex_lock(&ss->mutex);
 
-	old_files.da = ss->files.da;
-	ss->files.da = new_files.da;
+	old_files = ss->files;
+	ss->files = new_files;
 	if (new_tr) {
 		old_tr = ss->transition;
 		ss->transition = new_tr;
@@ -427,7 +429,7 @@ static void ss_update(void *data, obs_data_t *settings)
 
 	if (old_tr)
 		obs_source_release(old_tr);
-	free_files(&old_files.da);
+	free_files(&old_files);
 
 	/* ------------------------- */
 
@@ -659,8 +661,9 @@ static void ss_destroy(void *data)
 	struct slideshow *ss = data;
 
 	obs_source_release(ss->transition);
-	free_files(&ss->files.da);
+	free_files(&ss->files);
 	pthread_mutex_destroy(&ss->mutex);
+	calldata_free(&ss->cd);
 	bfree(ss);
 }
 
@@ -696,8 +699,13 @@ static void *ss_create(obs_data_t *settings, obs_source_t *source)
 		obs_module_text("SlideShow.PreviousSlide"),
 		previous_slide_hotkey, ss);
 
-	proc_handler_add(ph, "int current_index()", current_slide_proc, ss);
-	proc_handler_add(ph, "int total_files()", total_slides_proc, ss);
+	proc_handler_add(ph, "void current_index(out int current_index)",
+			 current_slide_proc, ss);
+	proc_handler_add(ph, "void total_files(out int total_files)",
+			 total_slides_proc, ss);
+
+	signal_handler_t *sh = obs_source_get_signal_handler(ss->source);
+	signal_handler_add(sh, "void slide_changed(int index, string path)");
 
 	pthread_mutex_init_value(&ss->mutex);
 	if (pthread_mutex_init(&ss->mutex, NULL) != 0)
@@ -802,9 +810,7 @@ static inline bool ss_audio_render_(obs_source_t *transition, uint64_t *ts_out,
 			float *out = audio_output->output[mix].data[ch];
 			float *in = child_audio.output[mix].data[ch];
 
-			memcpy(out, in,
-			       AUDIO_OUTPUT_FRAMES * MAX_AUDIO_CHANNELS *
-				       sizeof(float));
+			memcpy(out, in, AUDIO_OUTPUT_FRAMES * sizeof(float));
 		}
 	}
 
