@@ -8,6 +8,8 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <errno.h>
+#include <string.h>
 
 struct virtualcam_data {
 	obs_output_t *output;
@@ -75,8 +77,7 @@ static bool loopback_module_loaded()
 		}
 	}
 
-	if (fp)
-		fclose(fp);
+	fclose(fp);
 
 	return loaded;
 }
@@ -128,12 +129,12 @@ static bool try_connect(void *data, const char *device)
 		return false;
 
 	if (ioctl(vcam->device, VIDIOC_QUERYCAP, &capability) < 0)
-		return false;
+		goto fail_close_device;
 
 	format.type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
 
 	if (ioctl(vcam->device, VIDIOC_G_FMT, &format) < 0)
-		return false;
+		goto fail_close_device;
 
 	struct obs_video_info ovi;
 	obs_get_video_info(&ovi);
@@ -146,7 +147,7 @@ static bool try_connect(void *data, const char *device)
 	parm.parm.output.timeperframe.denominator = ovi.fps_num;
 
 	if (ioctl(vcam->device, VIDIOC_S_PARM, &parm) < 0)
-		return false;
+		goto fail_close_device;
 
 	format.fmt.pix.width = width;
 	format.fmt.pix.height = height;
@@ -154,7 +155,7 @@ static bool try_connect(void *data, const char *device)
 	format.fmt.pix.sizeimage = vcam->frame_size;
 
 	if (ioctl(vcam->device, VIDIOC_S_FMT, &format) < 0)
-		return false;
+		goto fail_close_device;
 
 	struct video_scale_info vsi = {0};
 	vsi.format = VIDEO_FORMAT_YUY2;
@@ -162,10 +163,23 @@ static bool try_connect(void *data, const char *device)
 	vsi.height = height;
 	obs_output_set_video_conversion(vcam->output, &vsi);
 
+	memset(&parm, 0, sizeof(parm));
+	parm.type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
+
+	if (ioctl(vcam->device, VIDIOC_STREAMON, &parm) < 0) {
+		blog(LOG_ERROR, "Failed to start streaming on '%s' (%s)",
+		     device, strerror(errno));
+		goto fail_close_device;
+	}
+
 	blog(LOG_INFO, "Virtual camera started");
 	obs_output_begin_data_capture(vcam->output, 0);
 
 	return true;
+
+fail_close_device:
+	close(vcam->device);
+	return false;
 }
 
 static int scanfilter(const struct dirent *entry)
@@ -226,8 +240,17 @@ static void virtualcam_stop(void *data, uint64_t ts)
 {
 	struct virtualcam_data *vcam = (struct virtualcam_data *)data;
 	obs_output_end_data_capture(vcam->output);
-	close(vcam->device);
 
+	struct v4l2_streamparm parm = {0};
+	parm.type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
+
+	if (ioctl(vcam->device, VIDIOC_STREAMOFF, &parm) < 0) {
+		blog(LOG_WARNING,
+		     "Failed to stop streaming on video device %d (%s)",
+		     vcam->device, strerror(errno));
+	}
+
+	close(vcam->device);
 	blog(LOG_INFO, "Virtual camera stopped");
 
 	UNUSED_PARAMETER(ts);
