@@ -515,6 +515,9 @@ static enum gs_color_space
 color_grade_filter_get_color_space(void *data, size_t count,
 				   const enum gs_color_space *preferred_spaces)
 {
+	UNUSED_PARAMETER(count);
+	UNUSED_PARAMETER(preferred_spaces);
+
 	const enum gs_color_space potential_spaces[] = {
 		GS_CS_SRGB,
 		GS_CS_SRGB_16F,
