@@ -1113,7 +1113,9 @@ bool obs_module_load(void)
 	si.get_properties = get_properties;
 	si.icon_type = OBS_ICON_TYPE_TEXT;
 
-	si.get_name = [](void *) { return obs_module_text("TextGDIPlus"); };
+	si.get_name = [](void *) {
+		return obs_module_text("TextGDIPlus");
+	};
 	si.create = [](obs_data_t *settings, obs_source_t *source) {
 		return (void *)new TextSource(source, settings);
 	};
@@ -1126,7 +1128,9 @@ bool obs_module_load(void)
 	si.get_height = [](void *data) {
 		return reinterpret_cast<TextSource *>(data)->cy;
 	};
-	si.get_defaults = [](obs_data_t *settings) { defaults(settings, 1); };
+	si.get_defaults = [](obs_data_t *settings) {
+		defaults(settings, 1);
+	};
 	si.update = [](void *data, obs_data_t *settings) {
 		reinterpret_cast<TextSource *>(data)->Update(settings);
 	};
