@@ -20,7 +20,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QCloseEvent>
 
 EditorWidget::EditorWidget(QWidget *parent, VSTPlugin *plugin)
-	: QWidget(parent), plugin(plugin)
+	: QWidget(parent),
+	  plugin(plugin)
 {
 	setWindowFlags(this->windowFlags() |= Qt::MSWindowsFixedSizeDialogHint);
 }
